{******************************************************************************}
{*                                                                            *}
{* Component:                                    *}
{* Author:                                                       *}
{*                                                                            *}
{* Email:mail@silentium.ru                                                    *}
{* htttp://www.silentium.ru                                                   *}
{* tel: +7-0912-25-17-50                                                      *}
{* Copyright (c) Silentium 2004                                               *}
{*                                                                            *}
{******************************************************************************}
{*                                                                            *}
{*      OnExecute                                    *}
{*    EditType:                                              *}
{*  etSimple -   ,                        *}
{*  etComplex - ,                                             *}
{*  etDate - ,                                   *}
{*  etFile - ,                                            *}
{*                                                                            *}
{******************************************************************************}
unit SiComboEdit;

interface

uses
  {$WARNINGS OFF}
  Classes, Controls, DBCtrls, Buttons, ExtCtrls, Forms, Graphics, Windows,
  Dialogs, Mask, SysUtils, StdCtrls, Messages,
  SiDateDlg, SiConst, SiUtils, FileCtrl;
  {$WARNINGS ON}
type
 //  
 TEditType = (etSimple, etComplex, etDate, etFile, etDirectory);

  //   
  TSiComboEdit = class (TPanel)
  private
    //   label'
    FLabelPosition :TLabelPosition;
    FBeforeChoose :TNotifyEvent; //     
    FAfterChoose :TNotifyEvent;  //     
    FExecute :TNotifyEvent;  //    
    FButton: TSpeedButton;
    FPrepared: Boolean;
    FEditType: TEditType;
    FEdit: TMaskEdit;
    FEnabled: Boolean;
    FInitialDir :String;
    FLabel :TLabel;
    //   FLabel  
    FLabelSpacing :integer;
    // 
    //FGlyph: Pointer;
    FFilter: string;
    FUseTime: Boolean;
    FShowen: Boolean;
    //  18.12.2009
    // FOpenDialogSample :TOpenDialog;
    procedure SetPrepared(const Value: Boolean);
    procedure SetEditType(const Value: TEditType);
    function GetText: string;
    procedure SetText(const Value: string);
    procedure SetEnabled(const Value: Boolean); reintroduce; virtual;
    function GetOnChange: TNotifyEvent;
    procedure SetOnChange(const Value: TNotifyEvent);
    //     control'
    procedure wm_move(var message :TMessage); message WM_MOVE;
    function GetCaption: string;
    procedure SetCaption(const Value: string);
    procedure SetLableSpacing(const Value: integer);
    procedure SetLablePosition(const Value: TLabelPosition);
    procedure Execute(Sender :TObject);
    //      
    procedure PanelEnter(Sender: TObject);
    procedure SetShowen(const Value: Boolean);
  public
    constructor Create(Owner: TComponent); override;
    //  
    procedure Prepare;
    //   
    procedure PanelResize(Sender: TObject);
    //  
    procedure RunDate(Sender: TObject);
    //   
    procedure RunFileOpen(Sender: TObject);
    //   
    procedure EditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    //   
    procedure RunDirSelect(Sender: TObject);
  published
    property Filter: string read FFilter write FFilter;
    //  
    property Edit: TMaskEdit read FEdit write FEdit;
    //  
    property Button: TSpeedButton read FButton write FButton;
    // 
    property Prepared: Boolean read  FPrepared write SetPrepared;
    //   
    property OnExecute: TNotifyEvent read FExecute write FExecute;
    //  
    property EditType: TEditType read FEditType write SetEditType;
    //  
    property Text: string read GetText write SetText;
    // 
    property Enabled: Boolean read FEnabled write SetEnabled;
    //    
    property OnChange: TNotifyEvent read GetOnChange write SetOnChange;
    property InitialDir :String Read FInitialDir Write FInitialDir;
    //  caption  FLabel
    property Caption :string read GetCaption write SetCaption;
    property LabelSpacing :integer read FLabelSpacing write SetLableSpacing;
    property BeforeChoose :TNotifyEvent read FBeforeChoose write FBeforeChoose;
    property AfterChoose :TNotifyEvent read FAfterChoose write FAfterChoose;
    property LabelPosition :TLabelPosition read FLabelPosition write
      SetLablePosition;
    //   
    // property Glyph: TBitmap read GetGlyph write SetGlyph;
    //   
    property UseTime: Boolean read FUseTime write FUseTime;  //  18.12.2009
    // 
    property Showen: Boolean read FShowen write SetShowen;
  protected
    //  .   
    procedure SetParent(AParent :TWinControl); override;
    //     FLabel
    procedure SetLabelPosition; virtual;
  end;

implementation

constructor TSiComboEdit.Create(Owner: TComponent);
begin
  inherited;
  FLabelPosition := lpLeft;
  FLabel := TLabel.Create(self);
  FLabelSpacing := 5;
  FLabel.Alignment := taRightJustify;
  BevelOuter := bvLowered;
  Height := SIDBCONTROL_EDIT_HEIGHT;
  Width := SIDBCONTROL_EDIT_WIDTH + SIDBCONTROL_EDIT_BTN_WIDTH;
  Constraints.MinHeight := SIDBCONTROL_EDIT_HEIGHT;
  Constraints.MinWidth := SIDBCONTROL_EDIT_BTN_WIDTH;
  OnResize := PanelResize;
  //  
  FEdit := TMaskEdit.Create(Self);
  with FEdit do
  begin
    Parent := Self;
    OnKeyDown := EditKeyDown;
    OnDblClick := Execute;
  end;
  Button := TSpeedButton.Create(Self);
  with Button do
  begin
    Width := SIDBCONTROL_EDIT_BTN_WIDTH;
    Parent := Self;
    Align := alRight;
    TabStop := False;
    OnClick := Execute;
//    Flat := true;
  end;
  //   
  OnEnter := PanelEnter;
  //  
  Enabled := true;
  FShowen := true;
  Prepare;
end;

procedure TSiComboEdit.Prepare;
begin
  PanelResize(Self);
end;

procedure TSiComboEdit.SetPrepared(const Value: Boolean);
begin
  FPrepared := Value;
  Prepare;
end;

//   
procedure TSiComboEdit.PanelResize(Sender: TObject);
begin
  with FEdit do
  begin
    Height := Self.Height;
    Width := Self.Width - SIDBCONTROL_EDIT_BTN_WIDTH;
  end;
end;

//  
procedure TSiComboEdit.RunDate(Sender: TObject);
var
  xDataDialog: TSiDateDialog;
begin
  xDataDialog := TSiDateDialog.Create(Self);
  try
    xDataDialog.Date := StrToDateDef(FEdit.Text, Date);
    xDataDialog.UseTime := FUseTime; // 18.12.2009
  except end;  
  if xDataDialog.Execute then
    Text := DateToStr(xDataDialog.Date);
    //Text := DateTimeToStr(xDataDialog.Date); // 18.12.2009
  xDataDialog.Free;
end;

//   
procedure TSiComboEdit.SetEditType(const Value: TEditType);
begin
  FEditType := Value;
  case FEditType of
    etSimple:
    begin
      //  Button.Glyph.LoadFromResourceName(HInstance, 'EDIT_NONE');
      //  FEdit.EditMask := '';
      //  FEdit.Color := clWindow;
      //  Button.OnClick := nil;
    end;
    etComplex:
    begin
      Button.Glyph.LoadFromResourceName(HInstance, 'EDIT_COMPLEX');
      FEdit.EditMask := '';
      FEdit.Color := clBtnFace;
      //Button.OnClick := nil;
    end;
    etDate:
    begin
      Button.Glyph.LoadFromResourceName(HInstance, 'EDIT_DATE');
    {  if FUseTime then       //  18.12.2009
        FEdit.EditMask := SIDBCONTROL_DATETIME_MASK
      else   }
        FEdit.EditMask := SIDBCONTROL_DATE_MASK;  
      FEdit.Color := clWindow;
    end;
    etFile:
    begin
      Button.Glyph.LoadFromResourceName(HInstance, 'EDIT_OPEN');
      FEdit.EditMask := '';
      FEdit.Color := clWindow;
    end;
    etDirectory:
    begin
      Button.Glyph.LoadFromResourceName(HInstance, 'EDIT_OPEN');
      FEdit.EditMask := '';
      FEdit.Color := clWindow;
    end;
  end;
end;

//    Ctrl+Enter
procedure TSiComboEdit.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ( Key = VK_RETURN ) and ( ssCtrl in Shift ) then
    Button.Click;
end;

//     
procedure TSiComboEdit.RunFileOpen(Sender: TObject);
var
  xOpenDialog: TOpenDialog;
begin
  xOpenDialog := TOpenDialog.Create(Self);
  xOpenDialog.InitialDir := FInitialDir;
  xOpenDialog.Filter := FFilter;
  xOpenDialog.FileName := Text;
  if xOpenDialog.Execute then
    Text := xOpenDialog.FileName;
  xOpenDialog.Free;
end;


function TSiComboEdit.GetText: string;
begin
  Result := FEdit.Text;
end;

procedure TSiComboEdit.SetText(const Value: string);
begin
  FEdit.Text := Value;
end;

procedure TSiComboEdit.SetEnabled(const Value: Boolean);
begin
  FEnabled := Value;
  FEdit.Enabled := FEnabled;
  FButton.Enabled := FEnabled;
  if FEnabled then
    FEdit.Color := clWindow
  else
    FEdit.Color := clBtnFace;
end;

function TSiComboEdit.GetOnChange: TNotifyEvent;
begin
  Result := FEdit.OnChange;
end;

procedure TSiComboEdit.SetOnChange(const Value: TNotifyEvent);
begin
  FEdit.OnChange := Value;
end;

procedure TSiComboEdit.RunDirSelect(Sender: TObject);
var
  xPath :String;
begin
  if SelectDirectory(' ', FInitialDir, xPath) then
    Text := xPath;
end;

procedure TSiComboEdit.wm_move(var message: TMessage);
begin
  inherited;
  SetLabelPosition;
end;

function TSiComboEdit.GetCaption: string;
begin
  Result := FLabel.Caption;
end;

procedure TSiComboEdit.SetCaption(const Value: string);
begin
  FLabel.Caption := Value;
  SetLabelPosition;  
end;

procedure TSiComboEdit.SetParent(AParent: TWinControl);
begin
  inherited;
  if AParent = nil then
    Exit;
  FLabel.Parent := Parent;
end;

procedure TSiComboEdit.SetLabelPosition;
begin
  case FLabelPosition of
    lpLeft : begin
      FLabel.Top := Top + 2;
      FLabel.Left := Left - FLabel.Width - FLabelSpacing;
    end;
    lpAbove : begin
      FLabel.Top := Top - FLabel.Height - FLabelSpacing;
      FLabel.Left := Left;
    end;
    lpBelow : begin
      FLabel.Top := Top + Height + FLabelSpacing;
      FLabel.Left := Left;
    end;
    lpRight : begin
      FLabel.Top := Top + 2;
      FLabel.Left := Left + Width + FLabelSpacing;
    end;
  end;  
end;

procedure TSiComboEdit.SetLableSpacing(const Value: integer);
begin
  FLabelSpacing := Value;
  SetLabelPosition;
end;

procedure TSiComboEdit.SetLablePosition(const Value: TLabelPosition);
begin
  FLabelPosition := Value;
  SetLabelPosition;
end;

procedure TSiComboEdit.Execute(Sender: TObject);
begin
  if @FBeforeChoose <> nil then FBeforeChoose(Sender);
  case FEditType of
   etDate : RunDate(Sender);
   etFile : RunFileOpen(Sender);
   etDirectory : RunDirSelect(Sender);
  else
    if @FExecute <> nil then FExecute(Sender);
  end;
  if @FAfterChoose <> nil then FAfterChoose(Sender);
end;

procedure TSiComboEdit.PanelEnter(Sender: TObject);
begin
  FEdit.SetFocus;
end;

procedure TSiComboEdit.SetShowen(const Value: Boolean);
begin
  FShowen := Value;
  FLabel.Visible  := FShowen;
  FEdit.Visible   := FShowen;
  FButton.Visible := FShowen;
  Self.Visible    := FShowen;
end;

end.
