{******************************************************************************}
{*                                                                            *}
{* Component:                                                  *}
{* Author:                                                       *}
{*                                                                            *}
{* Email:mail@silentium.ru                                                    *}
{* htttp://www.silentium.ru                                                   *}
{* tel: +7-0912-25-17-50                                                      *}
{* Copyright (c) Silentium 2004                                               *}
{*                                                                            *}
{******************************************************************************}
unit SiDateDlg;

interface

uses
  Windows, Classes, Graphics, Forms, Controls, Buttons, Math, ComCtrls,
  SysUtils, StdCtrls, Grids, Calendar, ExtCtrls, Spin, SiComPanel;

const
  ListWidth = 100;     //     
  CalendarSize = 226;  //  
  Space = 10;

type

  TCustomCalendar = class(TCalendar)
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  end;

  TfrmCalendar = class(TForm)  // 
  private
    FFormCaption: string;
    ComPanel: TSiComPanel;
    Calendar: TCustomCalendar;
    sedtYear: TSpinEdit;
    lbxMonth: TListBox;
    dtpTime: TDateTimePicker;
    procedure CalendarChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure sedtYearChange(Sender: TObject);
    procedure lbxMonthClick(Sender: TObject);
    procedure CalendarDblClick(Sender: TObject);
    procedure btnTodayClick(Sender: TObject);
    procedure SetChoiceDate(Value: TDateTime);
    function GetChoiceDate: TDateTime;
    procedure SetFormCaption(Value: string);
  published
    property ChoiceDate: TDateTime read GetChoiceDate write SetChoiceDate;
    property FormCaption: string read FFormCaption write SetFormCaption;
  public
    constructor CreateNew(AOwner: TComponent; Dummy: Integer = 0); override;
    procedure FormCreate(Sender: TObject);
    function TakeDate: Boolean;
  end;

  TSiDateDialog = class(TComponent)
  private
    FDate: TDate;
    FCaption: string;
    FfrmGetDate: TfrmCalendar;
    FUseTime: Boolean;
  published
    //   
    property UseTime: Boolean read FUseTime write FUseTime;
    // ,  
    property Date: TDate read FDate write FDate;
    //    
    property Caption: string read FCaption write FCaption;
  public
    constructor Create(AOwner: TComponent); override;
    function Execute: boolean;
  end;


implementation

procedure TCustomCalendar.KeyDown(var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_Prior: PrevMonth;
    VK_Next: NextMonth;
    VK_Home: PrevYear;
    VK_End: NextYear;
  else
    inherited;
  end;
end;

// 
constructor TfrmCalendar.CreateNew(AOwner: TComponent; Dummy: Integer = 0);
begin
  inherited;
  OnActivate := FormActivate;
end;

//  
procedure TfrmCalendar.FormCreate(Sender: TObject);
var
  InfoPanel: TPanel;
begin
  BorderStyle := bsDialog;
  InfoPanel := TPanel.Create(Self);
  with InfoPanel do
  begin
    Caption := '';
    Align := alClient;
    Parent := Self;
  end;
  sedtYear := TSpinEdit.Create(Self);
  with sedtYear do
  begin
    Parent := InfoPanel;
    Width := ListWidth;
    Left := Space;
    Top := Space;
    OnChange := sedtYearChange;
  end;

  lbxMonth := TListBox.Create(Self);
  with lbxMonth do
  begin
    Parent := InfoPanel;
    Width := ListWidth;
    Height := CalendarSize - sedtYear.Height - Space * 4;
    Left := Space;
    Top := sedtYear.Top + sedtYear.Height + Space;
    Items.Add('01. ');    //   
    Items.Add('02. ');
    Items.Add('03. ');
    Items.Add('04. ');
    Items.Add('05. ');
    Items.Add('06. ');
    Items.Add('07. ');
    Items.Add('08. ');
    Items.Add('09. ');
    Items.Add('10. ');
    Items.Add('11. ');
    Items.Add('12. ');
    OnClick := lbxMonthClick;
  end;

  dtpTime := TDateTimePicker.Create(Self);
  with dtpTime do
  begin
    Parent := InfoPanel;
    Width := ListWidth;
    Left := Space;
    Top := CalendarSize - dtpTime.Height + Space;
    Kind := dtkTime;
    Enabled := false;
    Color := clBtnFace;
  end;

  Calendar := TCustomCalendar.Create(Self);
  with Calendar do
  begin
    Parent := InfoPanel;
    Height := CalendarSize;
    Width := CalendarSize;
    Left := sedtYear.Left + sedtYear.Width + Space;
    Top := Space;
    OnChange := CalendarChange;
    OnDblClick := CalendarDblClick;
    StartOfWeek := 1;
  end;
  ComPanel := TSiComPanel.Create(Self);
  with ComPanel do
  begin
    Parent := Self;
    btnAll.Caption := '&';   // SelectAll  ""
    btnAll.ModalResult := mrNone;
    OnAllClick := btnTodayClick;  //
   btnAll.Visible := true;        //
  end;
  Position := poScreenCenter;
  ClientWidth := Calendar.Left + Calendar.Width + Space;
  ClientHeight := Max(Calendar.Top + Calendar.Height, lbxMonth.Top + lbxMonth.Height) + ComPanel.Height + Space;
end;

//  
procedure TfrmCalendar.SetChoiceDate(Value: TDateTime);
begin
  Calendar.CalendarDate := Value;
  dtpTime.Time := Value;
end;

//  
function TfrmCalendar.GetChoiceDate: TDateTime;
begin
  Result := Calendar.CalendarDate;
end;

//      
procedure TfrmCalendar.CalendarChange(Sender: TObject);
begin
  Caption := FFormCaption + ' - ' + DateToStr(Calendar.CalendarDate);
  sedtYear.Value := Calendar.Year;
  lbxMonth.ItemIndex := Calendar.Month - 1;
end;

//  
procedure TfrmCalendar.FormActivate(Sender: TObject);
begin
  sedtYear.Value := Calendar.Year;
  lbxMonth.ItemIndex := Calendar.Month - 1;
  Calendar.SetFocus;  //   
end;

//  
procedure TfrmCalendar.sedtYearChange(Sender: TObject);
begin
 with Calendar do
 begin
   if Day > DaysPerMonth(sedtYear.Value, Month) then
     Day := DaysPerMonth(sedtYear.Value, Month);
   Year := sedtYear.Value;
 end;
end;

//  
procedure TfrmCalendar.lbxMonthClick(Sender: TObject);
begin
 with Calendar do
 begin
   if Day > DaysPerMonth(Year, lbxMonth.ItemIndex + 1) then
     Day := DaysPerMonth(Year, lbxMonth.ItemIndex + 1);
   Month := lbxMonth.ItemIndex + 1;
 end;
end;

//    
procedure TfrmCalendar.CalendarDblClick(Sender: TObject);
begin
  Close;
  ModalResult := mrOk;
end;

//    ""
procedure TfrmCalendar.btnTodayClick(Sender: TObject);
begin
  Calendar.CalendarDate := Date;
  lbxMonth.ItemIndex := Calendar.Month - 1;
  sedtYear.Value := Calendar.Year;
end;

// !  
//    
//        ADate
//   ,    .
//    .
//   ,  true,     
//    ChoiceDate
function TfrmCalendar.TakeDate: Boolean;
begin
  ShowModal;
  if ModalResult = mrOk then
    Result := true
  else
    Result := false;
end;

procedure TfrmCalendar.SetFormCaption(Value: string);
begin
  FFormCaption := Value;
  Caption := FFormCaption + ' - ' + DateToStr(Calendar.CalendarDate);
end;

constructor TSiDateDialog.Create(AOwner: TComponent);
begin
  inherited;
  FfrmGetDate := TfrmCalendar.CreateNew(Self);
  FfrmGetDate.FormCreate(Self);
  FCaption := ' ';
  Date := 0;
end;

function TSiDateDialog.Execute: boolean;
var
  xTimeDate, xChiceDate: TSystemTime;
begin
  with FfrmGetDate do
  begin
    //   
    dtpTime.Enabled := FUseTime;
    if FUseTime then
      dtpTime.Color := clWindow
    else
      dtpTime.Color := clBtnFace;
    //   ,   
    if Date = 0 then
      ChoiceDate := SysUtils.Date
    else
      ChoiceDate := Date;
    FormCaption := FCaption;
    //     
    Result := TakeDate;
    if Result then
    begin
      //    -  ,  -  
      if not FUseTime then
        Date := ChoiceDate
      else
      begin
        DateTimeToSystemTime(dtpTime.Date, xTimeDate);
        DateTimeToSystemTime(ChoiceDate, xChiceDate);
        xChiceDate.wSecond := xTimeDate.wSecond;
        xChiceDate.wMinute := xTimeDate.wMinute;
        xChiceDate.wHour := xTimeDate.wHour;
        Date := SystemTimeToDateTime(xChiceDate);
      end;
    end;
  end;
end;

end.
