{******************************************************************************}
{*                                                                            *}
{* Component:   Ini-                                            *}
{* Author:                                                       *}
{*                                                                            *}
{* Email:mail@silentium.ru                                                    *}
{* htttp://www.silentium.ru                                                   *}
{* tel: +7-0912-25-17-50                                                      *}
{* Copyright (c) Silentium 2004                                               *}
{*                                                                            *}
{******************************************************************************}
unit SiIniFile;

interface

uses
  Classes, SysUtils, dialogs, IniFiles, Forms, StdCtrls, Graphics,
  SiUtils, SiComboEdit;

const
  //     
  SIINIFILE_MAXLINES = 100;
  //     ''
  SIINIFILE_STRINGDEFAULT = '';
  //   
  SIINIFILE_STRINGSCOUNT = '_Lines';
  //   
  SIINIFILE_STRINGSLINE = '_Line_';

  //  
  SIINIFILE_FONT_NAME  = '_FontName';
  //  
  SIINIFILE_FONT_STYLE = '_FontStyle';
  //  
  SIINIFILE_FONT_SIZE  = '_FontSize';
  //  
  SIINIFILE_FONT_COLOR = '_FontColor';

  SIINIFILE_FS_DELEMETER = ',';
  SIINIFILE_FS_BOLD      = 'fsbold';
  SIINIFILE_FS_ITALIC    = 'fsitalic';
  SIINIFILE_FS_UNDERLINE = 'fsUnderLine';
  SIINIFILE_FS_STRIKEOUT = 'fsstrikeout';



  // ,   
  SIINIFILE_QUOTE = '''';       SIINIFILE_QUOTE_CODE = '<QUOTE_0027>';
  //  ,  
  SIINIFILE_DOUBLE_QUOTE = '"'; SIINIFILE_QUOTE_DOUBLE_CODE = '<QUOTE_0022>';
  // 
  SIINIFILE_TAB = #9;           SIINIFILE_TAB_CODE = '<TAB_0015>';
  //  
  SIINIFILE_RETURN = #13#10;    SIINIFILE_RETURN_CODE = '<RETURN_0013>';
  // 
  SIINIFILE_SPACE = ' ';        SIINIFILE_SPACE_CODE = '<SPACE_0032>';

type
//  TSiIniFile = class(TIniFile)  //  . .
  TSiIniFile = class(TMemIniFile)
  private
    FCheckDefault: boolean;
    FFastSaving: Boolean;
    FSafeSaving: Boolean;
    FUpdating: Boolean;
    F_FileName: string;
  public
    // 
    constructor Create(const FileName: String);
    // 
    destructor Destroy; override;
    //  
    procedure WriteString(const Section, Ident, Value: String); override;
    //   ( 28.04.2010)
    procedure WriteInteger(const Section, Ident: string; Value: Longint); override;
    //    ( 28.04.2010)
    procedure WriteBool(const Section, Ident: string; Value: Boolean); override;
    //   
    procedure WriteStrings(ASectionName: string; AValueName: string;
      AStrings: TStrings; ADefaultValue: string = '');
    //    
    procedure WriteFormElements(ASection: string; AForm: TForm);
    //    ( 28.04.2010)
    procedure WriteBinaryStream(const Section, Name: string; Value: TStream); override;
    //   ( 28.04.2010)
    procedure WriteDate(const Section, Name: string; Value: TDateTime); override;
    //  - ( 28.04.2010)
    procedure WriteDateTime(const Section, Name: string; Value: TDateTime); override;
    //      ( 28.04.2010)
    procedure WriteFloat(const Section, Name: string; Value: Double); override;
    //   ( 28.04.2010)
    procedure WriteTime(const Section, Name: string; Value: TDateTime); override;
    //    ( 28.04.2010)
    procedure EraseSection(const Section: string); override;
    //   ( 28.04.2010)
    procedure DeleteKey(const Section, Ident: String); override;
    //   
    procedure ReadFormElements(ASection: string; AForm: TForm);
    //  
    procedure SetGenerator(ASection, Ident: string; Value: Integer);
    //  
    procedure WriteFont(ASection, Ident: string; AFont: TFont);
    //  
    function GetGenerator(ASection, Ident: String): Integer;
    //   
    function ReadStrings(ASectionName: string; AValueName: string): TStrings;
    //    
    function ReadLongStrings(ASectionName: string; AValueName: string): TStrings;
    //  
    function ReadString(const Section, Ident, Default: string): string; override;
    //    ( 2047 )
    function ReadLongString(const Section, Ident, Default: string): string;
    //  
    function ReadFont(ASection, Ident: String; ADefault: TFont): TFont;
    //    Default? (   )
    property CheckDefault: boolean read FCheckDefault write FCheckDefault;
    //  
    property FastSaving: Boolean read FFastSaving write FFastSaving;
    //   ( )
    property SafeSaving: Boolean read FSafeSaving write FSafeSaving;
    //    
    property Updating: Boolean read FUpdating write FUpdating;
  end;

implementation

{ TSiIniFile }

uses Windows;

constructor TSiIniFile.Create(const FileName: String);
var
  xFileHandle: Integer;
begin
  inherited Create(FileName);
  //  07.06.2010
  try
    //      
    if not FileExists(FileName) then
    begin
      xFileHandle := FileCreate(FileName);
      FileClose(xFileHandle);
    end;
  except
  end;
  FFastSaving := True;   // -  ,   
  //  ,  
  FSafeSaving := True;
  F_FileName := FileName;
  FUpdating := False;
//  FFastSaving := False;
end;

function TSiIniFile.ReadString(const Section, Ident,
  Default: string): string;
{var
  Buffer: array[0..65000] of Char;}
begin
  Result := inherited ReadString(Section, Ident, Default);
  if FSafeSaving then
  begin
    // ,   
    Result := SiChangeWords(Result, SIINIFILE_QUOTE_CODE, SIINIFILE_QUOTE);
    //  ,  
    Result := SiChangeWords(Result, SIINIFILE_QUOTE_DOUBLE_CODE, SIINIFILE_DOUBLE_QUOTE);
    // 
    Result := SiChangeWords(Result, SIINIFILE_TAB_CODE, SIINIFILE_TAB);
    //  
    Result := SiChangeWords(Result, SIINIFILE_RETURN_CODE, SIINIFILE_RETURN);
    // 
    Result := SiChangeWords(Result, SIINIFILE_SPACE_CODE, SIINIFILE_SPACE);
  end;
end;

procedure TSiIniFile.WriteString(const Section, Ident, Value: String);
var
  xValue: string;
begin
  xValue := Value;
  if FSafeSaving then
  begin
    // ,   
    xValue := SiChangeWords(xValue, SIINIFILE_QUOTE, SIINIFILE_QUOTE_CODE);
    //  ,  
    xValue := SiChangeWords(xValue, SIINIFILE_DOUBLE_QUOTE, SIINIFILE_QUOTE_DOUBLE_CODE);
    // 
    xValue := SiChangeWords(xValue, SIINIFILE_TAB, SIINIFILE_TAB_CODE);
    //  
    xValue := SiChangeWords(xValue, SIINIFILE_RETURN, SIINIFILE_RETURN_CODE);
    // 
    xValue := SiChangeWords(xValue, SIINIFILE_SPACE, SIINIFILE_SPACE_CODE);
  end;
  inherited WriteString(Section, Ident, xValue);
  FUpdating := True;
end;

function TSiIniFile.ReadStrings(ASectionName,
  AValueName: string): TStrings;
var
  i, xCount: integer;
  xDefaultValue: string;
begin
  Result := TStringList.Create;
  Result.Clear;
  xCount := ReadInteger(ASectionName, AValueName + SIINIFILE_STRINGSCOUNT, 0);
  xDefaultValue := ReadString(ASectionName, AValueName + SIINIFILE_STRINGDEFAULT, '');
  for i := 0 to xCount - 1 do
    Result.Add(ReadString(ASectionName, AValueName + SIINIFILE_STRINGSLINE + IntToStr(i), xDefaultValue));
end;

procedure TSiIniFile.WriteStrings(ASectionName, AValueName: string;
  AStrings: TStrings; ADefaultValue: string = '');
var
  i: integer;
begin
  if ExtractFileName(FileName) = '' then
    Exit;
  try
    FUpdating := True;
    if not FastSaving then
      for i := 0 to SIINIFILE_MAXLINES do
       DeleteKey(ASectionName, AValueName + SIINIFILE_STRINGSLINE + IntToStr(i));
    WriteInteger(ASectionName, AValueName + SIINIFILE_STRINGSCOUNT, AStrings.Count);
    WriteString(ASectionName, AValueName + SIINIFILE_STRINGDEFAULT, ADefaultValue);
    for i := 0 to AStrings.Count - 1 do
      //if AStrings[i] <> ADefaultValue then
      WriteString(ASectionName, AValueName + SIINIFILE_STRINGSLINE + IntToStr(i), AStrings[i]);
  except
    FUpdating := False;
  end;
end;


function TSiIniFile.ReadLongString(const Section, Ident,
  Default: string): string;
var
  Buffer: array[0..65535] of Char;
begin
  //   (      
  //   2047 
  SetString(Result, Buffer, GetPrivateProfileString(PChar(Section),
            PChar(Ident), PChar(Default), Buffer, SizeOf(Buffer), PChar(F_FileName)));
  if FSafeSaving then
  begin
    // ,   
    Result := SiChangeWords(Result, SIINIFILE_QUOTE_CODE, SIINIFILE_QUOTE);
    //  ,  
    Result := SiChangeWords(Result, SIINIFILE_QUOTE_DOUBLE_CODE, SIINIFILE_DOUBLE_QUOTE);
    // 
    Result := SiChangeWords(Result, SIINIFILE_TAB_CODE, SIINIFILE_TAB);
    //  
    Result := SiChangeWords(Result, SIINIFILE_RETURN_CODE, SIINIFILE_RETURN);
    // 
    Result := SiChangeWords(Result, SIINIFILE_SPACE_CODE, SIINIFILE_SPACE);
  end;
end;

function TSiIniFile.ReadLongStrings(ASectionName,
  AValueName: string): TStrings;
var
  i, xCount: integer;
  xDefaultValue: string;
begin
  Result := TStringList.Create;
  Result.Clear;
  xCount := ReadInteger(ASectionName, AValueName + SIINIFILE_STRINGSCOUNT, 0);
  xDefaultValue := ReadLongString(ASectionName, AValueName + SIINIFILE_STRINGDEFAULT, '');
  for i := 0 to xCount - 1 do
    Result.Add(ReadLongString(ASectionName, AValueName + SIINIFILE_STRINGSLINE + IntToStr(i), xDefaultValue));
end;

procedure TSiIniFile.WriteFormElements(ASection: string; AForm: TForm);
var
  i: LongInt;
  xControl: TComponent;
  xTxtValue: string;
begin
  for i := 0 to AForm.ComponentCount - 1 do
  begin
    xControl := AForm.Components[i];
    //   
    if xControl is TEdit then
    begin
      xTxtValue := TEdit(xControl).Text;
      WriteString(ASection, xControl.Name, xTxtValue);
    end;
    //    Silentium
    if xControl is  TSiComboEdit then
    begin
      xTxtValue := TSiComboEdit(xControl).Edit.Text;
      WriteString(ASection, xControl.Name, xTxtValue);
    end;
  end;
  FUpdating := True;
end;

procedure TSiIniFile.ReadFormElements(ASection: string; AForm: TForm);
var
  i: LongInt;
  xControl: TComponent;
  xTxtValue: string;
begin
  for i := 0 to AForm.ComponentCount - 1 do
  begin
    xControl := AForm.Components[i];
    //   
    if xControl is TEdit then
    begin
      xTxtValue := ReadString(ASection, xControl.Name, xTxtValue);
      TEdit(xControl).Text := xTxtValue;
    end;
    //    Silentium
    if xControl is  TSiComboEdit then
    begin
      xTxtValue := ReadString(ASection, xControl.Name, '');
      TSiComboEdit(xControl).Edit.Text := xTxtValue;
    end;
  end;
end;

function TSiIniFile.GetGenerator(ASection, Ident: String): Integer;
begin
  Result := ReadInteger(ASection, Ident, 1);
  WriteInteger(ASection, Ident, Result + 1);
end;

procedure TSiIniFile.SetGenerator(ASection, Ident: string; Value: Integer);
begin
  WriteInteger(ASection, Ident, Value);
  FUpdating := True;
end;

procedure TSiIniFile.WriteFont(ASection, Ident: string; AFont: TFont);
var
   xStyle: string;
begin
  //  
  WriteString(ASection, Ident + SIINIFILE_FONT_NAME, AFont.Name);
  //  
  xStyle := '';
  if fsBold in AFont.Style then
    xStyle := xStyle + SIINIFILE_FS_BOLD + SIINIFILE_FS_DELEMETER;
  if fsItalic in AFont.Style then
    xStyle := xStyle + SIINIFILE_FS_ITALIC + SIINIFILE_FS_DELEMETER;
  if fsUnderLine in AFont.Style then
    xStyle := xStyle + SIINIFILE_FS_UNDERLINE + SIINIFILE_FS_DELEMETER;
  if fsStrikeOut in AFont.Style then
    xStyle := xStyle + SIINIFILE_FS_STRIKEOUT + SIINIFILE_FS_DELEMETER;
  WriteString(ASection, Ident + SIINIFILE_FONT_STYLE, xStyle);    
  //  
  WriteInteger(ASection, Ident + SIINIFILE_FONT_SIZE, AFont.Size);
  //  
  WriteInteger(ASection, Ident + SIINIFILE_FONT_COLOR, AFont.Color);
  FUpdating := True;
end;

function TSiIniFile.ReadFont(ASection, Ident: String; ADefault: TFont): TFont;
var
  xStyle: string;
begin
  Result := ADefault;
  //  
  Result.Name := ReadString(ASection, Ident + SIINIFILE_FONT_NAME, ADefault.Name);
  //  
  xStyle := ReadString(ASection, Ident + SIINIFILE_FONT_STYLE, '');
  if Pos(SIINIFILE_FS_BOLD, xStyle) > 0 then
    Result.Style := Result.Style + [fsBold];
  if Pos(SIINIFILE_FS_ITALIC, xStyle) > 0 then
    Result.Style := Result.Style + [fsItalic];
  if Pos(SIINIFILE_FS_UNDERLINE, xStyle) > 0 then
    Result.Style := Result.Style + [fsUnderLine];
  if Pos(SIINIFILE_FS_STRIKEOUT, xStyle) > 0 then
    Result.Style := Result.Style + [fsStrikeOut];
  //  
  Result.Size := ReadInteger(ASection, Ident + SIINIFILE_FONT_SIZE, ADefault.Size);
  //  
  Result.Color := ReadInteger(ASection, Ident + SIINIFILE_FONT_COLOR, ADefault.Color);
end;


destructor TSiIniFile.Destroy;
begin
  //  28.04.2010
  try
    if FileExists(FileName) then
    begin
      //   ,   
      if FUpdating then
      begin
        UpdateFile;
        FUpdating := False;
      end;
    end;
  except
  end;
  inherited Destroy;
end;

procedure TSiIniFile.WriteInteger(const Section, Ident: string;
  Value: Integer);
begin
  inherited WriteInteger(Section, Ident, Value);
  FUpdating := True;
end;

procedure TSiIniFile.WriteBool(const Section, Ident: string;
  Value: Boolean);
begin
  inherited WriteBool(Section, Ident, Value);
  FUpdating := True;
end;

procedure TSiIniFile.WriteBinaryStream(const Section, Name: string;
  Value: TStream);
begin
  inherited WriteBinaryStream(Section, Name, Value);
  FUpdating := True;
end;

procedure TSiIniFile.WriteDate(const Section, Name: string;
  Value: TDateTime);
begin
  inherited WriteDate(Section, Name, Value);
  FUpdating := True;
end;

procedure TSiIniFile.WriteDateTime(const Section, Name: string;
  Value: TDateTime);
begin
  inherited WriteDateTime(Section, Name, Value);
  FUpdating := True;
end;

procedure TSiIniFile.WriteFloat(const Section, Name: string;
  Value: Double);
begin
  inherited WriteFloat(Section, Name, Value);
  FUpdating := True;
end;

procedure TSiIniFile.WriteTime(const Section, Name: string;
  Value: TDateTime);
begin
  inherited WriteTime(Section, Name, Value);
  FUpdating := True;
end;

procedure TSiIniFile.DeleteKey(const Section, Ident: String);
begin
  inherited DeleteKey(Section, Ident);
  FUpdating := True;
end;

procedure TSiIniFile.EraseSection(const Section: string);
begin
  inherited EraseSection(Section);
  FUpdating := True;
end;

end.



